@@ -6,12 +6,12 @@ from pathlib import Path
 from typing import Callable, Optional, Dict, Tuple
 
 from keras import Model  # type: ignore
-from onnx2kerastl.exceptions import OnnxUnsupported # type: ignore
+from onnx2kerastl.exceptions import OnnxUnsupported  # type: ignore
 from tensorflow.keras.models import load_model  # type: ignore
 import tarfile
 import ntpath
 from onnx2kerastl import onnx_to_keras  # type: ignore
-
+from leap_model_rebuilder import rebuild_model # type: ignore
 from keras_data_format_converter import convert_channels_first_to_last  # type: ignore
 
 from leap_model_parser.contract.importmodelresponse import NodeResponse, ImportModelTypeEnum
@@ -21,10 +21,25 @@ onnx_imported = False
 package_name = 'onnx'
 spec = find_spec(package_name)
 if spec is not None:
-    import onnx # type: ignore
+    import onnx  # type: ignore
 
     onnx_imported = True
 
+
+def get_k_model_from_pb_path(file_path):
+    tf = tarfile.open(file_path)
+    with tempfile.TemporaryDirectory() as temp_dir:
+        tf.extractall(temp_dir)
+        pb_files = glob.glob(temp_dir + "/**/*.pb", recursive=True)
+        if len(pb_files) == 0:
+            raise Exception('no pb files were found')
+
+        pb_file_path = next(iter(pb_files))
+        pb_folder_path = next(iter(ntpath.split(pb_file_path)))
+        k_model = load_model(pb_folder_path)
+    return k_model
+
+
 class ModelParser:
     def __init__(self):
         self._should_transform_inputs = False
@@ -37,16 +52,33 @@ class ModelParser:
         }
 
     def get_keras_model_and_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
-                                        should_transform_inputs=False) -> Tuple[
-        Dict[str, NodeResponse], Optional[Model]]:
+                                        should_transform_inputs=False) -> Tuple[Dict[str, NodeResponse], Optional[Model]]:
+
         self._should_transform_inputs = should_transform_inputs
         model_to_keras_converter: Optional[Callable[[str], Dict]] = self._model_types_converter.get(model_type.value)
         if model_to_keras_converter is None:
             raise Exception(f"Unable to import external version, {str(model_path)} file format isn't supported")
 
-        model_schema, keras_model = model_to_keras_converter(str(model_path))
-        model_generator = KerasJsonModelImport()
-        return model_generator.generate_graph(model_schema), keras_model
+        file_path = str(model_path)
+        try:
+            model_schema, keras_model = model_to_keras_converter(file_path)
+            model_generator = KerasJsonModelImport()
+            graph = model_generator.generate_graph(model_schema)
+            return graph, keras_model
+        except Exception as e:
+            if model_type.value in [ImportModelTypeEnum.H5_TF2.value, ImportModelTypeEnum.PB_TF2.value]:
+                if model_type.value == ImportModelTypeEnum.H5_TF2.value:
+                    keras_model = load_model(file_path)
+                else:
+                    keras_model = get_k_model_from_pb_path(file_path)
+
+                rebuilt_model = rebuild_model(keras_model)
+                model_schema, keras_model = self.convert_to_keras_model(rebuilt_model)
+                model_generator = KerasJsonModelImport()
+                graph = model_generator.generate_graph(model_schema)
+                return graph, keras_model
+            else:
+                raise e
 
     def generate_model_graph(self, model_path: Path, model_type: ImportModelTypeEnum,
                              should_transform_inputs=False) -> Dict[str, NodeResponse]:
@@ -60,18 +92,7 @@ class ModelParser:
         return model_schema, None
 
     def convert_pb_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
-        tf = tarfile.open(file_path)
-
-        with tempfile.TemporaryDirectory() as temp_dir:
-            tf.extractall(temp_dir)
-            pb_files = glob.glob(temp_dir + "/**/*.pb", recursive=True)
-            if len(pb_files) == 0:
-                raise Exception('no pb files were found')
-
-            pb_file_path = next(iter(pb_files))
-            pb_folder_path = next(iter(ntpath.split(pb_file_path)))
-            k_model = load_model(pb_folder_path)
-
+        k_model = get_k_model_from_pb_path(file_path)
         return self.convert_to_keras_model(k_model)
 
     def convert_onnx_model(self, file_path: str) -> Tuple[Dict[str, NodeResponse], Model]:
